#!/bin/sh -x

#
# Firmware update script
# Model: SX-582-2700
# Created: 2016.03.04
#
# $1 is the name of the tarball
#
TB=$1

# Remember where we started
WD=`pwd`

echo $WD

if [ -z "$TB" ] ; then
   echo -e "\nUsage: update.sh tarball"
   exit 1
fi

echo -e "\nFirmware update in progress. Do NOT power off!"

tar -jxf $TB blinkled
if [ ! -x blinkled ] ; then
   exit 1
fi

# Begin blinking the LED during the update
./blinkled
rm ./blinkled

# Kill off all applications that we will be updating
# (listed in alphabetical order)
killall -q \
        bbsd320 \
        console \
        fldpd \
        garpd \
        hostapd \
        hostapd_cli \
        inetd \
        inotifyd \
        jcpd \
        jcpd_uvl \
        ntpd \
        snmpd \
        sxbrd \
        thttpd \
        udhcpc \
        udhcpd \
        wpa_cli \
        wpa_supplicant

# Fetch all the files at once - faster than one at a time
tar -jxf $TB
#
# Prevent configuration files from being overwritten
#
CONFIG_FILES="
        /etc/init.d/dhcpscfg
        /etc/init.d/ipconfig
        /etc/shadow
        /etc/wpa_supplicant_null.conf
"
for i in $CONFIG_FILES; do
        if [ -e "$i" ]; then
                rm -f "tree$i"
        fi
done

# Prepare to replace www files with newer ones
rm -rf /usr/www/*

# Update files
cd tree
find . -type f -exec ../install.sh {} ../tree.md5/{}.md5 \;


#
# update busybox
#
busybox_updatefile() {
	cd $1
	md5sum -c -s $WD/$2.md5
	if [ $? -ne 0 ] ; then
		cd $WD
		$WD/$2 cp -f $2 $1/$2
		echo -e $1/$2 " updated"

		# update busybox symbolic links
                $1/$2 --install -s
                echo "New busybox installed"
	fi
	cd $WD
	rm -f $2 $2.md5
}

busybox_updatefile "/bin" "busybox"


#
# Confirm we have a valid kernel before erasing the old one
#
md5sum -c -s uImage.md5
if [ $? -eq 0 ] ; then
        echo -e "\nInstalling new kernel..."
        busybox flashcp -v uImage /dev/mtd2
        if [ $? -ne 0 ] ; then
                echo -e "\n!! Kernel installation failure !!"
                exit 0
        fi
         echo -e "\New kernel installed"
else
        echo -e "\nNew kernel image not valid. Update aborted"
        reboot
fi

#
# Flush flash changes and dismount file system
#
sync
sync
mount -r -o remount /

echo -e "\nFirmware update completed successfully. Restarting..."

#
# That's all folks!
#
reboot
